#!/bin/bash

# Copyright (C) 2013-2018 Centrify Corporation. All rights reserved.
#
function usage
{
    echo "usage: $0 target_user"
}

if [ $# -lt 1 ]; then
    usage
    exit 0
fi

TARGET_USER=$1

DM_PID=
DBUS_HANDLE=

CARD_EVENTMGR_CMD="/usr/share/centrifydc/bin/card_eventmgr nodaemon debug"

# exit if already launched
CARD_EVENTMGR_PROCESS=`ps ww -u $TARGET_USER -o uid,pid,cmd | grep "${CARD_EVENTMGR_CMD}$"`
if [ "$CARD_EVENTMGR_PROCESS""NO" != "NO" ]; then
    exit 0
fi

# keep trying for 30 seconds
START_TIME=$(date +%s)
while [ $(($(date +%s) - $START_TIME)) -lt 30 ]; do
    # wait 1 sec
    sleep 1

    # find display manager pid
    DM_PIDs=`ps ww -u $TARGET_USER -o uid,pid,cmd | egrep "kded4|gnome-session" | grep -v grep | awk '{ print $2 }' | xargs`
    if [ "x$DM_PIDs" = "x" ]; then
        continue
    fi

    DM_PID=`echo "$DM_PIDs" | awk '{ print $1 }'` 

    DBUS_HANDLE=`cat /proc/$DM_PID/environ | tr '\0' '\n' | grep DBUS`
    if [ "x$DBUS_HANDLE" = "x" ]; then
        continue
    fi
    break
done

# if $DBUS_HANDLE is empty, just assume we're not running KDE or GNOME, in which case we just exit
if [ -n "$DBUS_HANDLE" ]; then
    # launch card_eventmgr in background
    # - daemon flag doesn't seem to work
    su $TARGET_USER --shell /bin/bash -c "$DBUS_HANDLE $CARD_EVENTMGR_CMD &"

    # now wait till KDE or GNOME dies (user logs off)
    while kill -0 $DM_PID ; do
        sleep 1
    done

    # get the script's pid
    CARD_EVENTMGR_PIDS=`ps ww -u $TARGET_USER -o uid,pid,cmd | grep "$CARD_EVENTMGR_CMD" | grep -v grep | awk '{ print $2 }' | xargs`

    # kill the script
    if [ -n "$CARD_EVENTMGR_PIDS" ]; then
        kill $CARD_EVENTMGR_PIDS
    fi
fi
