#!/bin/bash
# Copyright (C) 2015-2018 Centrify Corporation. All rights reserved.
#
# Collect samba info in CDC pkg for adkeytab -C -m
#
# Parameter: basePath: samba's base path.
#            infoType: the type of samba info such as BuildInfo, ConfigInfo and VersionInfo.

PKG_NAME="CentrifyDC"
unset LD_PRELOAD LD_PRELOAD_32 LD_PRELOAD_64 LD_LIBRARY_PATH LD_LIBRARY_PATH_32 LD_LIBRARY_PATH_64 LIBPATH SHLIB_PATH DYND_LIBRARY_PATH DYLD_FALLBACK_LIBRARY_PATH
 
help_info(){

    echo "DESCRIPTION"
    echo "samba_info.sh is used to collect samba's versioninfo, buildinfo and configinfo"
    echo "SYNOPSIS"
    echo "samba_info.sh needs two arguments: samba's basePath and info type(such as: -b (buildinfo), -c (configinfo) and -v (versioninfo))."
}

if [ $# -ne 2 ]; then
    help_info
    exit 128
fi

basePath=$1
infoType=$2
case $infoType in
   -b)
       if [ -x "$basePath/sbin/smbd" ]; then
           $basePath/sbin/smbd -b  
       else  
           echo "$basePath/sbin/smbd does not exist or user does not have executable permissions"  
           exit 127 
       fi
       
       ;;
   -c)
       if [ -x "$basePath/bin/testparm" ]; then
           $basePath/bin/testparm -v -s 
       else 
           echo "$basePath/bin/testparm does not exist or user does not have executable permissions"
           exit 127
       fi 
       ;;
   -v)
       if [ -x "$basePath/bin/smbstatus" ]; then
           $basePath/bin/smbstatus -V
       else 
           echo "$basePath/bin/smbstatus does not exist or user does not have executable permissions"
           exit 127
       fi       
       ;;
   *)      
       echo "Unknown info type '$infoType', exiting"
       exit 128
       ;;

esac
