#!/bin/env adedit

#
# Copyright (C) 2015 Centrify Corporation. All rights reserved.
#
# This sample script is to create AD object (container or organizational unit)
# using adedit.
#
#
# Usage
# -----
#
# 1. Go to directory <centrifydc-install-path>/samples/hadoop/
#
# 2. Run the sample script, e.g.
# ./CreateADObject.tcl -d "OU=Hadoop,DC=example,DC=com"
# (OR adedit CreateADObject.tcl -d "OU=Hadoop,DC=example,DC=com")
#
# 3. Enter the following parameters interactively:
# - Domain name
# - AD account with administrative privileges
# - Password of the provided AD account
#
#
# Revision
# --------
#
# $Change: 319962 $
# $Revision: #1 $
# $DateTime: 2018/02/15 08:56:48 $
#
#

package require ade_lib

proc Usage { msg } {
    puts {usage: -d <object dn>}
    puts $msg
    exit 1
}

proc TypeToClass { dn } {
    if { [string match -nocase cn=* $dn] } {
        return "container"
    } else {
        return "organizationalUnit"
    }
}

proc EnsureExists { dn } {
    if { $dn == "" } { return }
    if { [catch {select_object $dn}] } {
        EnsureExists [get_parent_dn $dn]
        puts "Creating $dn..."
        new_object $dn
        sof objectClass [TypeToClass $dn]
        save_object
    }
}

if {[getopt argv -d dn] == 0} {
	Usage "Missing DN"
}

# Get domain and bind

puts "Enter domain name"
gets stdin domain 

set domaindn [dn_from_domain $domain]

puts "Enter account name with administrator privileges"
gets stdin administrator

puts "Enter $administrator password"
gets stdin APWD

bind $domain $administrator "$APWD"

puts "Creating AD object $dn if not exist yet..."
EnsureExists $dn
puts "AD object $dn created or already exist."
