################################################################################
#
# Copyright (C) 2015 Centrify Corporation. All rights reserved.
#
#
# Config subroutines for Hadoop sample scripts.
#
#
# Revision
# --------
#
# $Change: 319962 $
# $Revision: #1 $
# $DateTime: 2018/02/15 08:56:48 $
#
################################################################################

package CentrifyDC::Hadoop::Config;

use strict;
use warnings;

use CentrifyDC::GP::General qw(ReadFile);

use lib '/usr/share/centrifydc/samples/hadoop/perl';
use CentrifyDC::Hadoop::Logger qw(:debug_user :debug_verbose);

# >>> EXPORT >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

use base 'Exporter';

our @EXPORT_OK = qw(LoadUserConfig GetProperties);

our %EXPORT_TAGS = (
    'props'         => [qw(PROP_KEY_USE_COMP_OBJECT PROP_KEY_SERVICE_CONTAINER PROP_KEY_CLUSTER_SHORTNAME_ENABLE PROP_KEY_CLUSTER_SHORTNAME PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS PROP_KEY_SECURE_SHELL_USER_ENABLE PROP_KEY_SECURE_SHELL_USER PROP_KEY_SECURE_COPY_USER_ENABLE PROP_KEY_SECURE_COPY_USER PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE PROP_KEY_SECURE_SHELL_PRIVILEGE PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE)],
);

Exporter::export_ok_tags('props');

# >>> DATA >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# For subroutines GetPropString() and GetPropBool()
my $g_config;

#
# Properties
#

my $PROP_KEY_SERVICE_OBJECT_TYPE = 'hadoop.service.object.type';
my $PROP_DEF_SERVICE_OBJECT_TYPE = 'user';
my $PROP_KEY_SERVICE_CONTAINER = 'hadoop.service.container';
my $PROP_DEF_SERVICE_CONTAINER = 'cn=Users';
my $PROP_KEY_CLUSTER_SHORTNAME_ENABLE = "hadoop.cluster.shortname.enable";
my $PROP_DEF_CLUSTER_SHORTNAME_ENABLE = 1;
my $PROP_KEY_CLUSTER_SHORTNAME = 'hadoop.cluster.shortname';
my $PROP_DEF_CLUSTER_SHORTNAME = 'hdc1';
my $PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS = 'hadoop.host.shortname.extracted.last.chars';
my $PROP_DEF_HOST_SHORTNAME_EXTRACTED_LAST_CHARS = '2';
my $PROP_KEY_SECURE_SHELL_USER_ENABLE = 'hadoop.secure.shell.user.enable';
my $PROP_DEF_SECURE_SHELL_USER_ENABLE = 0;
my $PROP_KEY_SECURE_SHELL_USER = 'hadoop.secure.shell.user';
my $PROP_DEF_SECURE_SHELL_USER = "";
my $PROP_KEY_SECURE_COPY_USER_ENABLE = 'hadoop.secure.copy.user.enable';
my $PROP_DEF_SECURE_COPY_USER_ENABLE = 0;
my $PROP_KEY_SECURE_COPY_USER = 'hadoop.secure.copy.user';
my $PROP_DEF_SECURE_COPY_USER = "";
my $PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE = 'hadoop.secure.shell.privilege.enable';
my $PROP_DEF_SECURE_SHELL_PRIVILEGE_ENABLE = 0;
my $PROP_KEY_SECURE_SHELL_PRIVILEGE = 'hadoop.secure.shell.privilege';
my $PROP_DEF_SECURE_SHELL_PRIVILEGE = "";
my $PROP_KEY_SECURE_SHELL_KRB5CCNAME_ENABLE = 'hadoop.secure.shell.krb5ccname.enable';
my $PROP_DEF_SECURE_SHELL_KRB5CCNAME_ENABLE = 0;
my $PROP_KEY_SECURE_SHELL_KRB5CCNAME = 'hadoop.secure.shell.krb5ccname';
my $PROP_DEF_SECURE_SHELL_KRB5CCNAME = "";
my $PROP_KEY_SECURE_COPY_KRB5CCNAME_ENABLE = 'hadoop.secure.copy.krb5ccname.enable';
my $PROP_DEF_SECURE_COPY_KRB5CCNAME_ENABLE = 0;
my $PROP_KEY_SECURE_COPY_KRB5CCNAME = 'hadoop.secure.copy.krb5ccname';
my $PROP_DEF_SECURE_COPY_KRB5CCNAME = "";
my $PROP_KEY_DATA_DIR = 'hadoop.data.dir';
my $PROP_DEF_DATA_DIR = "/var/centrify/hadoop";
my $PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE = 'hadoop.adclient.krb5.service.principal.ftp.remove';
my $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE = 0;
my $PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE = 'hadoop.adclient.krb5.service.principal.cifs.remove';
my $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE = 0;
my $PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE = 'hadoop.adclient.krb5.service.principal.nfs.remove';
my $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE = 0;
my $PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE = 'hadoop.adclient.krb5.service.principal.http.remove';
my $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE = 1;

#
# Global variables
#

# Only properties defined in this hash will be updated by SetProperties()
our $g_props = {
    PROP_KEY_USE_COMP_OBJECT                        => '',
    PROP_KEY_SERVICE_CONTAINER                      => '',
    PROP_KEY_CLUSTER_SHORTNAME_ENABLE               => '',
    PROP_KEY_CLUSTER_SHORTNAME                      => '',
    PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS    => '',
    PROP_KEY_SECURE_SHELL_USER_ENABLE               => '',
    PROP_KEY_SECURE_SHELL_USER                      => '',
    PROP_KEY_SECURE_COPY_USER_ENABLE                => '',
    PROP_KEY_SECURE_COPY_USER                       => '',
    PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE          => '',
    PROP_KEY_SECURE_SHELL_PRIVILEGE                 => '',
    PROP_KEY_SECURE_SHELL_KRB5CCNAME_ENABLE         => '',
    PROP_KEY_SECURE_SHELL_KRB5CCNAME                => '',
    PROP_KEY_SECURE_COPY_KRB5CCNAME_ENABLE          => '',
    PROP_KEY_SECURE_COPY_KRB5CCNAME                 => '',
    PROP_KEY_DATA_DIR                               => '',
    PROP_KEY_DATA_SUB_DIR                           => '',
    PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE   => '',
    PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE  => '',
    PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE   => '',
    PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE  => '',
};

# >>> SUB >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#
# Load paramters from config file
#
#   $_[0]: config file path
#
#   return: hash ref of paramters loaded if succeeded
#           undef if failed
#
sub LoadConfig($)
{
    my $config_file = $_[0];
    my $properties = {};

    my $file_content = ReadFile($config_file);
    if (!defined($file_content))
    {
        ERROR_OUT("Failed to open config file $config_file to read");
        return undef;
    }

    my $skip = 0;
    my $tmp_line;

    my @lines = split(/\n/, $file_content);

    foreach (@lines)
    {
        next if (/^#|^$/);
        #
        # Join continuous line which ends with '\' to one line, then parse the joined line.
        #
        if (/\\$/)
        {
            $skip = 1;
            $tmp_line = $tmp_line.$_;
            $tmp_line =~ s/\\$//;
            chomp($tmp_line);
            next;
        }
        if ($skip)
        {
            $_ = $tmp_line.$_;
            $tmp_line = "";
            $skip = 0;
        }
        /([^\s:=]+)[:=]?\s*(.*)[\r\n]*$/;
        my ($property, $value) = ($1, $2);
        my @words = split('\s+', $value);
        $value = join(' ', @words);
        $properties->{$property} = $value;
    }

    #
    # Print config for debugging
    #
    
    VERBOSE_DEBUG("=== Config file content (comments removed) ===");

    foreach my $p (sort keys %$properties)
    {
        my $msg = "$p: $properties->{$p}";
        VERBOSE_DEBUG($msg);
    }

    VERBOSE_DEBUG("=== End of config file content (comments removed) ===");

    return $properties;
}

#
# Set properties
#
#   $_[0]: hash ref of the properties to set
#
#   return: 1       - succeeded
#           undef   - failed (e.g. $props is undef)
#
sub SetProperties($)
{
    my $props = $_[0];
    return undef if (!defined($props));

    # Only properties defined in $g_props will be updated
    foreach my $p (sort keys %$g_props)
    {
        $g_props->{$p} = $props->{$p};
    }

    return 1;
}

#
# Get properties
#
#   return: global hash ref which stores the properties
#
sub GetProperties()
{
    # Return hash ref $g_props for simplicity
    return $g_props;
}

sub GetPropString($$$)
{
    my $key = $_[0];
    my $def = $_[1];
    my $is_empty_allowed = $_[2];

    my $val = $g_config->{$key};

    return $def if (!defined($val));
    return $def if (($val eq "") && !$is_empty_allowed);
    return $val;
}

sub GetPropBool($$)
{
    my $key = $_[0];
    my $def = $_[1];

    my $val = $g_config->{$key};

    return $def if (!defined($val));

    return 1 if ($val =~ /true/i);
    return 0;
}

#
# Load user config and 
#
#   $_[0]: config file path
#
#   return: 1       - success
#           undef   - failure (e.g. $props is undef)
#
sub LoadUserConfig($)
{
    my $conf_file = $_[0];

    #
    # Load config file
    #

    $g_config = LoadConfig($conf_file);
    return undef if (!defined($g_config));

    #
    # Sanitize input and set properties
    #

    my $props = {};

    my $service_object_type = GetPropString($PROP_KEY_SERVICE_OBJECT_TYPE, $PROP_DEF_SERVICE_OBJECT_TYPE, 0);
    $props->{PROP_KEY_USE_COMP_OBJECT} = 0;
    $props->{PROP_KEY_USE_COMP_OBJECT} = 1 if ($service_object_type =~ /computer/);

    $props->{PROP_KEY_SERVICE_CONTAINER} = GetPropString($PROP_KEY_SERVICE_CONTAINER, $PROP_DEF_SERVICE_CONTAINER, 0);

    $props->{PROP_KEY_CLUSTER_SHORTNAME_ENABLE} = GetPropBool($PROP_KEY_CLUSTER_SHORTNAME_ENABLE, $PROP_DEF_CLUSTER_SHORTNAME_ENABLE);
    $props->{PROP_KEY_CLUSTER_SHORTNAME} = GetPropString($PROP_KEY_CLUSTER_SHORTNAME, $PROP_DEF_CLUSTER_SHORTNAME, 0);
    $props->{PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS} = GetPropString($PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS, $PROP_DEF_HOST_SHORTNAME_EXTRACTED_LAST_CHARS, 0);

    $props->{PROP_KEY_SECURE_SHELL_USER_ENABLE} = GetPropBool($PROP_KEY_SECURE_SHELL_USER_ENABLE, $PROP_DEF_SECURE_SHELL_USER_ENABLE);
    $props->{PROP_KEY_SECURE_SHELL_USER} = GetPropString($PROP_KEY_SECURE_SHELL_USER, $PROP_DEF_SECURE_SHELL_USER, 0);
    $props->{PROP_KEY_SECURE_COPY_USER_ENABLE} = GetPropBool($PROP_KEY_SECURE_COPY_USER_ENABLE, $PROP_DEF_SECURE_COPY_USER_ENABLE);
    $props->{PROP_KEY_SECURE_COPY_USER} = GetPropString($PROP_KEY_SECURE_COPY_USER, $PROP_DEF_SECURE_COPY_USER, 0);

    $props->{PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE} = GetPropBool($PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE, $PROP_DEF_SECURE_SHELL_PRIVILEGE_ENABLE);
    $props->{PROP_KEY_SECURE_SHELL_PRIVILEGE} = GetPropString($PROP_KEY_SECURE_SHELL_PRIVILEGE, $PROP_DEF_SECURE_SHELL_PRIVILEGE, 0);

    $props->{PROP_KEY_SECURE_SHELL_KRB5CCNAME_ENABLE} = GetPropBool($PROP_KEY_SECURE_SHELL_KRB5CCNAME_ENABLE, $PROP_DEF_SECURE_SHELL_KRB5CCNAME_ENABLE);
    $props->{PROP_KEY_SECURE_SHELL_KRB5CCNAME} = GetPropString($PROP_KEY_SECURE_SHELL_KRB5CCNAME, $PROP_DEF_SECURE_SHELL_KRB5CCNAME, 0);
    $props->{PROP_KEY_SECURE_COPY_KRB5CCNAME_ENABLE} = GetPropBool($PROP_KEY_SECURE_COPY_KRB5CCNAME_ENABLE, $PROP_DEF_SECURE_COPY_KRB5CCNAME_ENABLE);
    $props->{PROP_KEY_SECURE_COPY_KRB5CCNAME} = GetPropString($PROP_KEY_SECURE_COPY_KRB5CCNAME, $PROP_DEF_SECURE_COPY_KRB5CCNAME, 0);

    # DO NOT use /etc/security/keytabs for data sub-directory. Since otherwise
    # other keytab files might be overrided.
    $props->{PROP_KEY_DATA_DIR} = GetPropString($PROP_KEY_DATA_DIR, $PROP_DEF_DATA_DIR, 0);
    $props->{PROP_KEY_DATA_SUB_DIR} = "$props->{PROP_KEY_DATA_DIR}/$props->{PROP_KEY_CLUSTER_SHORTNAME}";

    $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE} = GetPropBool($PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE, $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE);
    $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE} = GetPropBool($PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE, $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE);
    $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE} = GetPropBool($PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE, $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE);
    $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE} = GetPropBool($PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE, $PROP_DEF_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE);

    SetProperties($props);

    # Get the global hash ref
    $props = GetProperties();

    #
    # Print parameters for debugging
    #

    VERBOSE_DEBUG("=== Configurable parameters ===");

    VERBOSE_DEBUG("Service account use AD computer object: $props->{PROP_KEY_USE_COMP_OBJECT}");
    VERBOSE_DEBUG("Service container: $props->{PROP_KEY_SERVICE_CONTAINER}");
    VERBOSE_DEBUG("Cluster shortname enable: $props->{PROP_KEY_CLUSTER_SHORTNAME_ENABLE}");
    VERBOSE_DEBUG("Cluster shortname: $props->{PROP_KEY_CLUSTER_SHORTNAME}");
    VERBOSE_DEBUG("Host shortname extracted last # chars: $props->{PROP_KEY_HOST_SHORTNAME_EXTRACTED_LAST_CHARS}");
    VERBOSE_DEBUG("Secure shell user enabled: $props->{PROP_KEY_SECURE_SHELL_USER_ENABLE}");
    VERBOSE_DEBUG("Secure shell user: $props->{PROP_KEY_SECURE_SHELL_USER}");
    VERBOSE_DEBUG("Secure copy user enabled: $props->{PROP_KEY_SECURE_COPY_USER_ENABLE}");
    VERBOSE_DEBUG("Secure copy user: $props->{PROP_KEY_SECURE_COPY_USER}");
    VERBOSE_DEBUG("Secure shell privilege enabled: $props->{PROP_KEY_SECURE_SHELL_PRIVILEGE_ENABLE}");
    VERBOSE_DEBUG("Secure shell privilege: $props->{PROP_KEY_SECURE_SHELL_PRIVILEGE}");
    VERBOSE_DEBUG("Secure shell KRB5CCNAME enabled: $props->{PROP_KEY_SECURE_SHELL_KRB5CCNAME_ENABLE}");
    VERBOSE_DEBUG("Secure shell KRB5CCNAME: $props->{PROP_KEY_SECURE_SHELL_KRB5CCNAME}");
    VERBOSE_DEBUG("Secure copy KRB5CCNAME enabled: $props->{PROP_KEY_SECURE_COPY_KRB5CCNAME_ENABLE}");
    VERBOSE_DEBUG("Secure copy KRB5CCNAME: $props->{PROP_KEY_SECURE_COPY_KRB5CCNAME}");
    VERBOSE_DEBUG("Data directory: $props->{PROP_KEY_DATA_DIR}");
    VERBOSE_DEBUG("Data sub-directory: $props->{PROP_KEY_DATA_SUB_DIR}");
    VERBOSE_DEBUG("Remove ftp SPN from AD computer objects: $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_FTP_REMOVE}");
    VERBOSE_DEBUG("Remove cifs SPN from AD computer objects: $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_CIFS_REMOVE}");
    VERBOSE_DEBUG("Remove nfs SPN from AD computer objects: $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_NFS_REMOVE}");
    VERBOSE_DEBUG("Remove http SPN from AD computer objects: $props->{PROP_KEY_ADCLIENT_KRB5_SERVICE_PRINCIPAL_HTTP_REMOVE}");

    VERBOSE_DEBUG("=== End of configurable parameters ===");

    return 1;
}

1;
