#!/bin/sh
# 
# Sample script to handle local accounts, 
# and is designed to be invoked by UNIX Agent to handle any 
# changes on the local accounts.  
#
# The sample script also illustrate the options that UNIX agent
# expect the notification CLI to have. 
# 
# Set this script to /etc/centrifydc/centrifydc.conf in  
# adclient.local.account.notification.cli, ie 
# adclient.local.account.notification.cli: /usr/share/centrifydc/samples/localacctmgmt/handle_local_accts.sh
# 
# Once set, UNIX Agent will invoke this notification script 
# where there are: 
#   - new local users added  or/and
#   - local users are locked or/and 
#   - local users are unlocked or/and
#   - local users are removed
#
# The sample script, for:  
#   - New local users: set random password, create home directory
#   - unlocked local users: set random password 
#   - removed and locked users: display users that removed and locked
#

# Set Paths
CIMS_USR=/usr
CIMS_INSTALL=/usr/share/centrifydc

usage() {
    echo -e $"Usage: $0 <-c, --computer <computer's samaccountname>>"
    echo -e $"                                [-z, --zone  <zone dn>]"
    echo -e $"                                [-n, --new  <name list>]"
    echo -e $"                                [-l, --locked  <name list>]"
    echo -e $"                                [-u, --unlocked <name list>]"
    echo -e $"                                [-r, --removed <name list>]"
    echo -e $"Options:"
    echo -e $"  -c, --computer <computer's samaccountname>\n      This is a required option, to specify the samaccount name of currently joined computer.\n"
    echo -e $"  -z, --zone <current zone DN>\n      The current zone DN that the computer joined to.\n"
    echo -e $"  -n, --new <comma separated list of unixname>\n      UNIX Agent will provide a list of comma separated unixname that are newly added, and is not locked.\n"
    echo -e $"  -l, --locked <comma separated list of unixname>\n      UNIX Agent will provide a list of comma separated unixname that are added as locked acct, or which account just changed from enabled to locked.\n"
    echo -e $"  -u, --unlocked <comma separated list of unixname>\n      UNIX Agent will provide a list of comma separated unixname which account just changed from locked to enabled.\n"
    echo -e $"  -r, --removed <comma separated list of unixname>\n      UNIX Agent will provide a list of comma separated unixname which account just removed.\n"
    echo -e $"  -h, --help\n      Print this help information and exit.\n\n"
    echo -e $"Return code:\n   0: on success\n   1: usage error\n   2: on other failure"    
}

setpassword() {
    PASSWORD=`$CIMS_INSTALL/bin/openssl rand -base64 8`
    USERNAME=$1

    if [ $TARGET_OS = "Linux" ] || [ $TARGET_OS = "AIX" ]; then
        echo $USERNAME:$PASSWORD | chpasswd >> $RESULT_FILE 2>&1
    elif [ $TARGET_OS = "SunOS" ]; then
        # Haven't found a good way to set a random password for user.
        echo "Note: Please better set a password for local user <$USERNAME>." >> $RESULT_FILE
    elif [ $TARGET_OS = "HP-UX" ]; then   
        /usr/sam/lbin/usermod.sam -F -p $PASSWORD $USERNAME >> $RESULT_FILE 2>&1
    else
        # For other platforms which have not been supported.
        echo "WARNING: Unknown platform \"$TARGET_OS\", set password utility unknown, unable to set password for user \"$USERNAME\"" >> $RESULT_FILE 2>&1
    fi
}

create_home_dir()
{
    DIR=""
    USERNAME=$1
    if [ $TARGET_OS = "SunOS" ]; then
        DIR="/export/home/$USERNAME"
    else
        DIR="/home/$USERNAME"
    fi
    
    if [ ! -d $DIR ];then
        mkdir -p $DIR
        chmod 0700 $DIR
        # Get gid of the user
        GID=`grep "^$USERNAME:" /etc/passwd | cut -d : -f 4 | sed 1q`
        [ "x$GID" = "x" ] && return 1
        
        
        # Copy files from skeleton directory (/etc/skel/) to the home directory, and change the files and directory ownership
        cp -R /etc/skel/. $DIR
        chown -R $USERNAME:$GID $DIR
    fi
}

umask 077

COMPUTERSAMNAME=""
ZONEDN=""
NEWADDEDLIST=""
LOCKEDLIST=""
UNLOCKEDLIST=""
NEWREMOVEDLIST=""
ret=0

# Set up a default search paths
PATH=/bin:/usr/bin:/sbin:/usr/sbin

# Prepare a tmp file as the result file
TMPDIR=`echo get_temp_dir | $CIMS_USR/bin/adedit`
RESULT_FILE="${TMPDIR}/notificationcli_result.txt"

echo "==================== begin ====================" >> $RESULT_FILE

TARGET_OS=`uname`

while getopts c:z:n:l:u:r:h: option
do

    case "$option" in
        c)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE
            COMPUTERSAMNAME=$OPTARG
            ;;
        z)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE
            ZONEDN=$OPTARG
            ;;
        n)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE
            NEWADDEDLIST=$OPTARG
            ;;
        l)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE        
            LOCKEDLIST=$OPTARG
            ;;
        u)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE        
            UNLOCKEDLIST=$OPTARG
            ;;
        r)
			echo "The option is : $option" >> $RESULT_FILE
			echo "The value is:   $OPTARG" >> $RESULT_FILE        
            NEWREMOVEDLIST=$OPTARG
            ;;
        h|*)
            usage
            exit 1
    esac
done

if [ -z "$COMPUTERSAMNAME" ]; then
    usage
    echo "Usage error! Computer's samaccountname can not be empty." >> $RESULT_FILE
    exit 1
else
    if [ ! -z "$ZONEDN" ]; then
        echo "Zone DN                : $ZONEDN"
        echo "Zone DN                : $ZONEDN" >> $RESULT_FILE
    fi
    
    if [ ! -z "$NEWADDEDLIST" ]; then
        echo "Newly add user list    : $NEWADDEDLIST" >> $RESULT_FILE
        OLD_IFS=$IFS
        IFS=,
       
        for UNIX_NAME in $NEWADDEDLIST
        do           
            # create home directory
            create_home_dir $UNIX_NAME
            ret=$?
            if [ "$ret" -ne 0 ]; then
                echo "Failed to create home directory for $UNIX_NAME" >> $RESULT_FILE
            fi
            
            # set a random password for it
            setpassword $UNIX_NAME
            ret=$?
            if [ "$ret" -ne 0 ]; then
                echo "Failed to set a random password for $UNIX_NAME" >> $RESULT_FILE
            fi
        done
       
        IFS=$OLD_IFS
    fi
    
    if [ ! -z "$LOCKEDLIST" ]; then
        echo "Locked user list       : $LOCKEDLIST" >> $RESULT_FILE
    fi
    
    if [ ! -z "$UNLOCKEDLIST" ]; then
        echo "Unlocked user list     : $UNLOCKEDLIST" >> $RESULT_FILE
        OLD_IFS=$IFS
        IFS=,
       
        for UNIX_NAME in $UNLOCKEDLIST
        do            
            # set a random password for it
            setpassword $UNIX_NAME
            ret=$?
            if [ "$ret" -ne 0 ]; then
                echo "Failed to set a random password for $UNIX_NAME" >> $RESULT_FILE
            fi            
        done
       
        IFS=$OLD_IFS
    fi
    
    if [ ! -z "$NEWREMOVEDLIST" ]; then
        echo "Newly removed user list: $NEWREMOVEDLIST" >> $RESULT_FILE
    fi
fi

echo "==================== end ====================" >> $RESULT_FILE
