#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2018 Centrify Corporation. All rights reserved.
#
# a package maitaining some common functions used by other 
# /cli/configurefiles/* perl scripts
#
############################################################################## 

package autoedit_common_utils;

use strict;
use warnings;

use base 'Exporter';
our @EXPORT_OK = qw(getValueList loadSetting insert_cfg_lines flushNameService);

my $NAME_SERVICE_RESTART_SCRIPT = '/usr/share/centrifydc/bin/nscdrestart.sh';

# read attribute from centrifydc.conf
# $name     the attribute name
# return    a Ref of array if found
#           0 if not found
#           undef, if meet anything wrong
sub getValueList
{
    my ($name) = @_;
    my $fileName = "/etc/centrifydc/centrifydc.conf";
    my $fileHandler;
    unless (open($fileHandler, "<$fileName"))
    {
        warn "cannot open $fileName : $!";
        return undef;
    }

    my $values = 0;
    foreach my $line (<$fileHandler>)
    {
        if ($line =~ /^\s*$name\s*:\s*(.*)/)
        {
            $values = [split /\s+/, $1];
            last;
        }
    }
    close($fileHandler);
    return $values;
}

# read attribute from kset files
# $name     the attribute name
# return    a Ref of array if found
#           undef, if meet anything wrong
sub loadSetting
{
    my ($name) = @_;
    my $fileName = "/var/centrifydc/kset.$name";
    my $fileHandler;
    unless(open($fileHandler, "<$fileName"))
    {
        warn "cannot open $fileName : $!";
        return undef;
    }

    my @values = ();
    foreach my $line (<$fileHandler>)
    {
        chomp($line);
        push(@values, $line);
    }
    close($fileHandler);
    return \@values;
}

# a basic function to insert our code into pam file.
# It just do two things:
#
# 1. Tell it which service or module you want,
# it will find the first line of the service/module,
# and insert our new code just before it.
#
# 2. Not all pam libs like our code to be above them.
# If find any one, we add our code after them.
#
# $cfgLines         the content of file, get from <FILE>
# $newLines         Centrifydc code to add before service/module
# $service          the service name we want, 
#                   It can be empty, that means any service
# $module           the module we want
# $libs_excluded    the pam libs who don't like our code 
#                   to be above them
# $newLines2        Centrifydc code, which will be insert 
#                   after service/module if found $libs_excluded
sub insert_cfg_lines
{
    my ($cfgLines, $newLines, $service, $module, 
        $libs_excluded, $newLines2) = @_;

    my $svc = defined $service ? $service : "";
    my $mdl = defined $module ? $module : "";

    # locate the first line of the service/module, which we are looking for
    my @wanted_idx;
    if (defined $service || defined $module)
    {
        my $wanted_lines = "^\\s*$svc\\s*$mdl\\s+";
        # if found the wanted, save its line number
        @wanted_idx = grep {$cfgLines->[$_] =~ /$wanted_lines/} 0..$#$cfgLines;
    }

    my $line_num = 0;
    if (defined $wanted_idx[0])
    {
        $line_num = $wanted_idx[0];
    }

    # have any pam libs don't like our code to be above them ?
    # need add our code behind them
    my @excluded_idx = ();
    if (defined $libs_excluded)
    {
        my $exls = "(".(join '|', @$libs_excluded).")";
        # who don't like us ?
        my $excluded_lines = "^\\s*$svc\\s*$mdl\\s+.*\\b$exls\\b";
        @excluded_idx = grep {$cfgLines->[$_] =~ /$excluded_lines/} $line_num..$#$cfgLines;
    }

    if (scalar(@excluded_idx) > 0)
    {
        # insert our code behind
        $line_num = $excluded_idx[$#excluded_idx] + 1;
    }

    if (scalar(@excluded_idx) == 0)
    {
        if (defined $newLines)
        {
            splice @$cfgLines, $line_num, 0, @$newLines;
            return @$newLines;
        }
    }
    else
    {
        if (defined $newLines2)
        {
            splice @$cfgLines, $line_num, 0, @$newLines2;
            return @$newLines2;
        }
    }
    return ();
}

# Function to flush name service cache.
# It is done by executing nscdrestart.sh script in centrifydc.
# The shell script does not return any status.
# 
sub flushNameService
{
    if (-e $NAME_SERVICE_RESTART_SCRIPT)
    {
        system("$NAME_SERVICE_RESTART_SCRIPT flush");
    }
    else
    {
        warn "WARNING: $NAME_SERVICE_RESTART_SCRIPT not found, name service cache not flushed";
    }
}

1;
