#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2018 Centrify Corporation. All rights reserved.
#
# process pam.d password-auth or common-auth style files
#
# arguments:
#       please see autoedit_configure.pm
# return:
#       0 if succeed
#       1 if meet error
#
##############################################################################

BEGIN{push(@INC, "/usr/share/centrifydc/scripts")};

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;
use warnings;

use autoedit_pam_d qw(process_pam_d_file process_pam_d_files);
use autoedit_configure qw($cdc_long_comment $cfg_dir $cdc_pam_lib
                          $libs_excluded $newLines $newLines2 $pam_d_ignored
                          $pam_d_extra @pam_d_common_confs);

# process pam.d password-auth or common-auth style files
#
# $cdc_pam_lib      the name of Centrifydc pam lib
# $libs_excluded    the pam libs, which do not like our code to be above them
#                   It is a Ref of array
# $cfg_dir          the full path of pam.d directory
# $cdc_long_comment Centrifydc comment added before its code
# $newLines         our new code to add before $service_module
# $newLines2        our new code to add behind $service_module if found $libs_excluded
# $pam_d_ignored    the pam.d files we ignored
# $pam_d_extra      the pam.d files that we will check again.
#                   Add centrifydc code, if it is not
#                   password-auth or common-auth style.
# return            undef if have anything wrong
#                   1 if succeed
#                   
sub process_pam_d_stack
{
    my ($cdc_pam_lib, $libs_excluded, $cfg_dir, $cdc_long_comment, 
        $newLines, $newLines2, $pam_d_ignored, $pam_d_extra) = @_;

    my $ret = 1;

    my @critical_files = @pam_d_common_confs;
    # need full path
    grep s/.*/$cfg_dir\/$_/, @critical_files;

    my $rc = process_pam_d_files(\@critical_files, $cdc_pam_lib,
                $libs_excluded, $cfg_dir, $cdc_long_comment,
                $newLines, $newLines2, \@pam_d_common_confs);
    # got error or did not find any pam.d file ?
    if (!$rc)
    {
        $ret = $rc;
    }

    # TBD: Explicitly ignore dzdo-* files for cloud

    # update "su" file
    my $cfg_file = "$cfg_dir/su";
    if (-e $cfg_file && !(grep /^$cfg_file$/, @$pam_d_ignored))
    {
        my @newLines  = ("auth\tsufficient\t$cdc_pam_lib enable_dzpamgate\n");
        $rc = process_pam_d_file($cfg_file, \@newLines, $libs_excluded, 
                    \@newLines, $cdc_pam_lib, $cdc_long_comment, undef);
        if (!$rc)
        {
            # meet error
            $ret = $rc;
        }
    }

    # update $pam_d_extra
    foreach my $file (@$pam_d_extra)
    {
        if (-e $file)
        {
            $rc = process_pam_d_file($file, $newLines, $libs_excluded,
                        $newLines2, $cdc_pam_lib, $cdc_long_comment, \@pam_d_common_confs);
            if (!$rc)
            {
                # meet error
                $ret = $rc;
            }
        }
    }
    return $ret;
}

my $ret = process_pam_d_stack($cdc_pam_lib, $libs_excluded, "$cfg_dir/pam.d", 
                $cdc_long_comment, $newLines, $newLines2, 
                $pam_d_ignored, $pam_d_extra);

exit ($ret ? 0 : 1);

